#include <Arduino.h>
#include <Adafruit_SSD1306.h>

class Game {
  private:
    int oledWidth, oledHeight;
    int ballPos[2];
    int ballDir[2];
    
    int batPos[2][2];
    int batHeight;
    int batButtons[2][2];

    Adafruit_SSD1306 * display;

    enum DIRECTION {NONE, UP, DOWN};
    

    void setInitialPos() {
      ballPos[0] = oledWidth / 2;
      ballPos[1] = oledHeight / 2;
      ballDir[0] = rand() % 2;
      ballDir[1] = rand() % 2;
      
      batPos[0][0] = 0;
      batPos[1][0] = oledWidth - 1;

      batPos[0][1] = batPos[1][1] = (oledHeight / 2) - (batHeight / 2);
    }

  public:
    Game(Adafruit_SSD1306 * d, int w, int h, int pinBtnLeftDown, int pinBtnLeftUp, int pinBtnRightDown, int pinBtnRightUp, int seed) {
      display = d;
      oledWidth = w;
      oledHeight = h;
      batHeight = oledHeight / 4;
      srand(seed);

      batButtons[0][0] = pinBtnLeftDown;
      batButtons[0][1] = pinBtnLeftUp;
      batButtons[1][0] = pinBtnRightDown;
      batButtons[1][1] = pinBtnRightUp;

      for(int x = 0; x < 2; x++) {
        for(int y = 0; y < 2; y++) {
          pinMode(batButtons[x][y], INPUT_PULLUP);
        }
      }
      
      setInitialPos();
    }

    void startNewGame() {
      setInitialPos();
    }

    bool checkForFail(int batId) {
      return !(ballPos[1] >= batPos[batId][1] && ballPos[1] <= batPos[batId][1] + batHeight);
    }

    void calcBall() {
      for(int x = 0; x < 2; x++) {
        if(ballDir[x]) {
          ballPos[x]++;
        } else {
          ballPos[x]--;
        }
      }

      if(ballPos[0] <= 0) {
        if(checkForFail(0)) {
          startNewGame();
          return;
        }
        ballDir[0] = !ballDir[0];
      } else if(ballPos[0] >= oledWidth - 1) {
        if(checkForFail(1)) {
          startNewGame();
          return;
        }
        ballDir[0] = !ballDir[0];
      }


      if(ballPos[1] <= 0 || ballPos[1] >= oledHeight - 1) {
          ballDir[1] = !ballDir[1];
      }
    }

    enum DIRECTION getBatDir(int btnDown, int btnUp) {
      if(!digitalRead(btnDown) && !digitalRead(btnUp)) return NONE;
      if(!digitalRead(btnDown)) return DOWN;
      if(!digitalRead(btnUp)) return UP;
      return NONE;
    }

    void calcBats() {
      enum DIRECTION dir;
      for(int x = 0; x < 2; x++) {
        dir = getBatDir(batButtons[x][0], batButtons[x][1]);
        if(dir == UP) {
          batPos[x][1]--;
        } else if(dir == DOWN) {
          batPos[x][1]++;
        }

        if(batPos[x][1] < 0) batPos[x][1] = 0;
        if(batPos[x][1] > oledHeight - batHeight) batPos[x][1] = oledHeight - batHeight;
      }
    }

    void draw() {
      display -> clearDisplay();
      display -> drawFastVLine(batPos[0][0], batPos[0][1], batHeight, 1);
      display -> drawFastVLine(batPos[1][0], batPos[1][1], batHeight, 1);
      display -> drawPixel(ballPos[0], ballPos[1], 1);
      display -> display();
    }
};

